/**
	\file 		ServoCalibrator.h
	\author 	Dustin Spicuzza: last changed by $Author: dspicuzz $
	\date 		Last changed on $Date: 2009-03-09 22:04:32 -0400 (Mon, 09 Mar 2009) $
	\version 	$Rev: 292 $
*/

#ifndef SERVOCALIBRATOR_H
#define SERVOCALIBRATOR_H


#include "RobotChassis.h"
#include "KwarqsLib/KwarqsJoystick.h"
#include "KwarqsLib/DriverStationLCD.h"
#include "KwarqsLib/DelayEvent.h"

#include "KwarqsLib/filters.h"

class ServoCalibrator {
public:

	ServoCalibrator(RobotChassis * chassis);
	
	void DoAutoCalibrate();
	void DoManualCalibrate();
	
	void DoLFServo();
	void DoLRServo();
	void DoRFServo();
	void DoRRServo();
	
	void Reset();

private:
	
	void DoServo(KwarqsWheelServo &servo);

	void ShowLCDOutput();

	KwarqsJoystick 		m_stick;
	RobotChassis * 		m_chassis;
	DriverStation * 	m_ds;
	DriverStationLCD * 	m_lcd;

	DelayEvent m_lcdEvent;
	DelayEvent m_triggerEvent;
	
	AverageWindowFilter<double, 20> m_potfilter1;
	AverageWindowFilter<double, 20> m_potfilter2;
	AverageWindowFilter<double, 20> m_potfilter3;
	AverageWindowFilter<double, 20> m_potfilter4;
	
	double pot1offset;
	double pot2offset;
	double pot3offset;
	double pot4offset;
	
	bool m_begin_manual_calibrate;
	
	bool last_trig;
};

#endif
